namespace :orientdb do
  desc "Orient DB integration tasks"

  task migrate: :environment do
    desc "Migrate orient db schema"

    migrations = OrientDb::Migrations.new(OrientDb::Client.instance)
    migrations.run
  end

  task create_learning_object_relations: :environment do
    desc "Create LearningObject relations based on its metadata"

    lo_repo = Portalmec::Application.repository.for(:learning_object)
    subject_repo = Portalmec::Application.repository.for(:subject)
    attr_repo = Portalmec::Application.repository.for(:attribute)

    # Quantity of LearningObjects fetched on each iteration
    limit = 1000
    # Starting point from where LearningObjects will be fetched
    offset = 0

    loop do
      p " --> importing LearningObjects from #{offset} to #{offset+limit}"

      begin
        # Get LearningObjects from OrientDB (from offset to offset+limit)
        learning_objects = lo_repo.all_from_offset_to_limit(offset,limit)
      rescue
        # Sleeps for a while to wait database's recovery
        sleep(30.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more LearningObjects
        break if learning_objects.empty?

        learning_objects.each do |lo|
          metadata = get_unique_matadata_keys(lo)
          subjects = metadata["dc.subject.category"]
          subjects ||= []
          metadata.delete("dc.subject.category")
          lo.subjects=[]
          lo.attributes=[]

          subjects.each do |subject_name|
            subject = subject_repo.find_by_name(subject_name)
            if subject.nil?
              subject = Subject.new(:name => subject_name)
              subject_repo.create(subject)
            end
            lo.subjects << subject
          end

          metadata.each do |key, value|
            value.each do |v|
              attribute = attr_repo.find_by_key_and_value(key,v).first
              if attribute.nil?
                attribute = Attribute.new(:key => key, :value => v)
                attr_repo.create(attribute)
              end
              lo.attributes << attribute
            end
          end
            
          p lo_repo.create_relations(lo)
        end
        
        offset += limit
      end
    end

  end

  def get_unique_matadata_keys(lo)
    hash = {}
    lo.metadata.each do |m|
      unless hash.has_key?(m['key'])
        hash[m['key']] = lo.get_metadata_values_of(m['key'])
      end
    end
    hash
  end

end